function [z1,z2,z3,cz1,cz2] = CreateHyperbolicKite(a,b,c)
%% computes the first hyperbolic kite with parameters a,b,c

lambda = (cos(pi/a) * cos(pi/b) + cos(pi/c))/(sin(pi/a)*sin(pi/b));
mu     = lambda + sqrt(lambda^2 - 1);
G      = (mu^2-1)/(2*(cot(pi/a) + mu*cot(pi/b)));

% number of segments approximating the arcs of the kite
segments = 20;

t = linspace(0,1,segments);

% edge1 is the lower arc of the kite
edge1 = arc((mu-1)/(mu+1),w(G/2+1i*f2(G/2,a,b,mu)),w(G+1i*f1(G,a,b,mu)),segments);
% edge2 is the upper arc of the kite
edge2 = conj(fliplr(edge1));
% edge4 is the lower line of the kite
edge4 = fliplr(t*w(G+1i*f1(G,a,b,mu)));
% edge3 is the upper line of the kite
edge3 = conj(fliplr(edge4));
% edge5 is the diagonal of the kite
edge5 = linspace(0,(mu-1)/(mu+1),200);

% mapping the kite to the upper half-plane
z1 = w_inv(edge4);
z2 = w_inv(edge1);
z3 = w_inv(edge5);
cz1 = w_inv(edge3);
cz2 = w_inv(edge2);
end 

function res = arc(A,B,C,n)
% Computes a circular arc containing the points A,B,C approximated by a
% polygonal chain of length n (counter clock-wise)

xA = real(A); xB = real(B); xC = real(C);
yA = imag(A); yB = imag(B); yC = imag(C);
 
Matrix = [yB-yC, -yA+yB; xC-xB, xA-xB];
b = 1/2*[xA-xC;yA-yC];
sol = Matrix\b;
tau = sol(1);

center = 1/2*(xB+xC)+tau*(yB-yC)+1i*(1/2*(yB+yC)+tau*(xC-xB));
radius = abs(B-center);

phiA = mod(angle(A-center),2*pi);
phiC = mod(angle(C-center),2*pi);

tAC = linspace(phiA,phiC,n);
res = center + radius*exp(1i*tAC);
end

function y = f1(x,a,~,~)
    y = sqrt((csc(pi/a))^2*ones(size(x)) - (x-cot(pi/a)).^2);
end

function y= f2(x,~,b,mu)
    y = sqrt(mu^2*csc(pi/b)^2*ones(size(x)) - (x + mu*cot(pi/b)).^2);
end

function res = w(z)
% maps upper half-plane to the unit disc via z -> (z-i)/(z+i)
    res = (z-1i) ./ (z+1i);
end

function res = w_inv(z)
% inverse of w
    res = 1i*(1+z) ./ (1-z);
end