% The following code is extracted from the weld.m file available at  
% https://github.com/oelarnes/treeweld

function [zz,poles] =  Welding(z,zz,poles,L)

z = exp(pi/30*1i)*z;
poles = exp(pi/30*1i)*poles;
zz = exp(pi/30*1i)*zz;

n = size(z,1)/2;

% new degree data for welding
d = ones(2 * n, 1);

z = (z - 1) ./ (z + 1);
zz = (zz - 1) ./ (zz + 1);
poles = (poles - 1) ./ (poles + 1);

z = [z; 1];

%% the welding loop
for j = 1:size(L,1)-1
    % i1,i2,i3,i4 are the indices of the endpoints of the next interval to
    % be welded
    i1 = L(j, 1);
    i2 = mod(L(j, 1)-2, 2 * n) + 1;
    i3 = L(j ,2);
    i4 = mod(L(j, 2)-2, 2 * n) + 1;
    % the values on the circle
	x1 = z(i1);
	x2 = .5 * z(i2) + .5 * z(i3);
	x3 = z(i4);
	% the angle to evenly space the new vertex
	alpha = d(i4) / (d(i4) + d(i1));
	% perform the weld
	z = alpha_moeb(z, x1, x2, x3, alpha);
    zz = alpha_moeb(zz, x1, x2, x3, alpha);
    poles = alpha_moeb(poles, x1, x2, x3, alpha);
	z = slit_map(z, alpha);
	zz = slit_map(zz, alpha);  
 	poles = slit_map(poles, alpha);
	% degree of new vertex
	d(i1) = d(i1) + d(i4);
	d(i4) = d(i1);
end

% back to the unit circle
z = (z + 1) ./ (z - 1);
zz = (zz + 1) ./ (zz - 1);
poles = (poles + 1) ./ (poles - 1);

i1 = L(end, 1);
i2 = mod(L(end, 1)-2, 2 * n) + 1;
i3 = L(end ,2);
i4 = mod(L(end, 2)-2, 2 * n) + 1;

x1 = .5 * z(i1) + .5 * z(i4);
x2 = .5 * z(i2) + .5 * z(i3);

% two remaining endpoints map to -1, 1
z = (x1 + x2 + 1i * (z - x1) * abs(x1 + x2) - x1 * z * conj(x1 + x2)) ./ ...
    (x1 + x2 - 1i * (z - x1) * abs(x1 + x2) - x1 * z * conj(x1 + x2));

zz = (x1 + x2 + 1i * (zz - x1) * abs(x1 + x2) - x1 * zz * conj(x1 + x2)) ./ ...
    (x1 + x2 - 1i * (zz - x1) * abs(x1 + x2) - x1 * zz * conj(x1 + x2));

poles = (x1 + x2 + 1i * (poles - x1) * abs(x1 + x2) - x1 * poles * conj(x1 + x2)) ./ ...
    (x1 + x2 - 1i * (poles - x1) * abs(x1 + x2) - x1 * poles * conj(x1 + x2));

% weld the circle
z = z + ones(size(z)) ./ z;
zz = zz + ones(size(zz)) ./ zz;
poles = poles + ones(size(poles)) ./ poles;

x = z(2 * n + 1);

% normalize by resetting infinity
zz = ones(size(zz)) ./ (zz - x);
poles = ones(size(poles)) ./ (poles - x);
end

function value = slit_map(z, alpha)
% slit map used for welding
value = (z+1i*alpha).^(alpha) .* (z+1i*(alpha-1)).^(1-alpha);
end

function value = alpha_moeb(z, x1, x2, x3, alpha)
% alpha_moeb resets the points on the imaginary axis so that
% the points x1, x2, x3 map to i(1-alpha), 0, -i*alpha.
value = moeb(z,x1,x2,x3,-1i*(alpha-1),0,-1i*alpha);
end

function res = moeb(z,z1,z2,z3,w1,w2,w3)
% computes the image of z of a Moebius transformation mapping [z1,z2,z3] to [w1,w2,w3] 
res = (w1*(-w2*(z1 - z2)*(z - z3*ones(size(z))) + w3*(z - z2*ones(size(z)))*(z1 - z3)) - ...
  w2* w3* (z - z1*ones(size(z)))* (z2 - z3))./(w3* (z1 - z2)* (z - z3*ones(size(z))) - ...
  w2* (z - z2*ones(size(z)))* (z1 - z3) + w1* (z - z1*ones(size(z)))* (z2 - z3));
end